/******************************************************************************
 * Join_Command - Hook up a player to an invited game.
 * $Id: Join_Command.java,v 1.20 2011/06/19 04:41:21 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Join_Command.java,v $
 * Revision 1.20  2011/06/19 04:41:21  dick
 * Send all of the FIBS resuming match messages to the players/watchers/others.
 *
 * Revision 1.19  2011/06/18 06:33:24  dick
 * Stop passing "dice" and "turn" all around.  In an MVC model, this board has all of that information already.
 * Use the board as an MVC document.
 * Align our messages outputted to be in the order that FIBS sends them.
 *
 * Revision 1.18  2011/06/16 19:04:40  dick
 * Crawford if test cleanup.
 *
 * Revision 1.17  2011/06/13 00:38:25  dick
 * whitespace.
 *
 * Revision 1.16  2011/06/11 19:18:35  dick
 * Stop passing jibsServer all around.  If anybody wants it, they can do JibsServer.getInstance().
 *
 * Revision 1.15  2011/06/11 04:59:04  dick
 * [sg]et[po]Die[12] becomes [sg]et[XO]Die[12].
 *
 * Revision 1.14  2011/06/09 18:59:57  dick
 * greet(Board) just takes a board and don't informAll on new game, only new match.
 *
 * Revision 1.13  2011/06/09 18:11:00  dick
 * Use Player.println() instead of the JibsWriter.
 * Fix crawford again.
 * Don't allow double on a 1 point match.
 *
 * Revision 1.12  2011/06/08 20:52:38  dick
 * Start match/game cleanups.
 * player[12] becomes player[XO].
 *
 * Revision 1.11  2011/06/07 20:16:03  dick
 * Fix crawford/postcrawford handling.
 *
 * Revision 1.10  2011/06/06 03:18:26  dick
 * JibsMessages belongs in n.s.jibs.server, not n.s.jibs.gui
 *
 * Revision 1.9  2011/06/06 02:52:17  dick
 * getAllClients() comes from JibsServer now.
 * Reset player toggles to new game state.
 *
 * Revision 1.8  2011/06/05 18:26:33  dick
 * log is a method of JibsServer.  Always write screen logs to the log file.
 * So, no need to pass write-to-logs=true from everywhere.
 *
 * Revision 1.7  2011/06/05 06:01:03  dick
 * Use less parameters in game constructors.  We don't need playerX and playerXName, that is redundant.
 *
 * Revision 1.6  2011/06/03 17:04:07  dick
 * getPlayerXPoints() becomes getPlayerXScore().
 *
 * Revision 1.5  2011/06/03 04:15:49  dick
 * Attributes player[12]Got becomes player[12]Score.
 *
 * Revision 1.4  2011/06/02 19:29:16  dick
 * When a player starts a game, he is no longer a watcher.
 *
 * Revision 1.3  2011/06/01 20:12:26  dick
 * informNewGame() tells everybody about the new match, not just the players.
 *
 * Revision 1.2  2011/05/31 13:02:43  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import net.sourceforge.jibs.backgammon.BackgammonBoard;
import net.sourceforge.jibs.backgammon.Die;
import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.backgammon.JibsMatch;
import net.sourceforge.jibs.server.JibsMessages;
import net.sourceforge.jibs.server.JibsQuestion;
import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.ClipConstants;
import net.sourceforge.jibs.util.InviteQuestion;
import net.sourceforge.jibs.util.JibsNewGameData;
import net.sourceforge.jibs.util.ResumeQuestion;
import net.sourceforge.jibs.util.SavedGameParam;

/** Hook up a player to an invited game.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.20 $ <br> $Date: 2011/06/19 04:41:21 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Join_Command.java">cvs Join_Command.java</a>
 */
public class Join_Command implements JibsCommand {
	private JibsServer jibsServer;

	/** Execute the <code>join</code> command.
	 */
	@Override
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		this.jibsServer = jibsServer;

		JibsGame game = player.getGame();
		Player opponent = null;
		int matchLength = 0;
//		JibsMatch matchVersion = null;

		if (game != null) {
			BackgammonBoard board = game.getBackgammonBoard();

			if (board.isEnded()) {
				return doJoinForNewMatch(player, args);
			} else {
				if (!board.isEnded()) {
					// Start the next game in a match
					opponent = board.getOpponent(player);
					matchLength = board.getMatchlength();
//					matchVersion = board.getMatchVersion();
					board.setJoinPlayer(player);

					if ((board.getJoinPlayer1() != null) &&
							(board.getJoinPlayer2() != null)) {
						// start next game
//						BackgammonBoard nextBoard = null;
//						nextBoard = new BackgammonBoard(jibsServer,
//								board.getPlayerX(),
//								board.getPlayerO(),
//								matchLength,
//								matchVersion);
//						nextBoard.setPlayerXScore(board.getPlayerXScore());
//						nextBoard.setPlayerOScore(board.getPlayerOScore());
						board.setInitialPositions();
						boolean useCrawford = player.checkToggle("crawford")
											  && opponent.checkToggle("crawford");
						if (board.isCrawFordGame())
							board.setPostCrawFordGame(true);
						else {
							if (useCrawford && (!board.isPostCrawfordGame() || matchLength == 1)) {
								int diffX = 0;
								int diffO = 0;

								diffX = board.getMatchlength() - board.getPlayerXScore();
								diffO = board.getMatchlength() - board.getPlayerOScore();

								if (diffX == 1 || diffO == 1) {
									board.setCrawFordGame(true);
									board.setMayDoubleO(0);
									board.setMayDoubleX(0);
								}
							}
						}
//						board = nextBoard;
//						game.setBackgammonBoard(board);

						greet(board);
						// XXX: This JibsNewGameData is either underused, or a waste.  There seems to be too many different ways to start a game.
						// what if it's not his turn to roll?  Could be dice on the table already.
						JibsNewGameData jibsNewGameData = jibsNewGameData(game.getPlayerX(),
								game.getPlayerO());
						board.setTurn(jibsNewGameData.getTurn());
						if (jibsNewGameData.getTurn() == BackgammonBoard.turnX) {
							board.setXDie1(jibsNewGameData.getDie1());
							board.setXDie2(jibsNewGameData.getDie2());
						} else {
							board.setODie1(jibsNewGameData.getDie1());
							board.setODie2(jibsNewGameData.getDie2());
						}

						board.setJoinPlayer1(null);
						board.setJoinPlayer2(null);
						player.setTogglesForNewGame();
						opponent.setTogglesForNewGame();
						game.startGame();
/*						player.startGame(game,

								jibsNewGameData.getTurn(),
								board.getMayDoubleX(),
								board.getMayDoubleO());
*/
					}
				}
			}
		} else {
			return doJoinForNewMatch(player, args);
		}
		return true;
	}

	private boolean doJoinForNewMatch(Player player, String[] args) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		Player opponent = null;
		int matchlength = 0;
		JibsMatch matchVersion = null;
		JibsGame game = player.getGame();
		String	msg;
		Object[] obj;
		if (args.length <= 1) {
			// m_join_no_user=** Error: Join who?
			msg = jibsMessages.convert("m_join_no_user");
			player.println(msg);
		} else {
			opponent = jibsServer.getPlayer(args[1]);

			if (opponent == null) {
				// m_join_no_invite=** %0 didn't invite you.
				obj = new Object[] { args[1] };
				msg = jibsMessages.convert("m_join_no_invite", obj);
				player.println(msg);
			} else {
				player.setOpponent(opponent);
				opponent.setOpponent(player);

				JibsQuestion question = opponent.getQuestion();

				if (question instanceof ResumeQuestion) {
					clipHasChanged(player, opponent);

					ResumeQuestion resumeQuestion = (ResumeQuestion) question;
					SavedGameParam resumeData = resumeQuestion.getResumeData();
					JibsGame resumeGame = new JibsGame(jibsServer, resumeData);
					BackgammonBoard board = resumeGame.getBackgammonBoard();
					resumeGame.setJibsServer(jibsServer);

					int startturn = board.getTurn();
					Die dice1 = board.getXDie1();
					Die dice2 = board.getXDie2();
					int player1Score = board.getPlayerXScore();
					int player2Score = board.getPlayerOScore();

					player.setGame(resumeGame);
					opponent.setGame(resumeGame);
					player.setTogglesForNewGame();
					opponent.setTogglesForNewGame();
					resumeGame.resumeGame(resumeData, startturn, dice1, dice2,
							player1Score, player2Score);
					// m_join_resume=%0 has joined you. Your running match was loaded.
					obj = new Object[] { opponent.getName() };
					msg = jibsMessages.convert("m_join_resume", obj);
					player.println(msg);
					player.show2Watchers(msg);
					// m_accept_resume=You are now playing with %0. Your running match was loaded.
					obj = new Object[] { player.getName() };
					msg = jibsMessages.convert("m_accept_resume", obj);
					opponent.println(msg);
					opponent.show2Watchers(msg);
					
					// m_resume_turn=turn: %0.
					obj = new Object[] { 
						startturn == BackgammonBoard.turnX
							? board.getPlayerXName() : board.getPlayerOName()
					};
					msg = jibsMessages.convert("m_resume_turn", obj);
					showToPlayersWatchers(player, opponent, msg);
					
					if (board.getMatchVersion().getVersion() == JibsMatch.unlimitedMatch) {
						// m_resume_length_unlimited=unlimited matchlength
						msg = jibsMessages.convert("m_resume_length_unlimited");
						showToPlayersWatchers(player, opponent, msg);
					} else {
						// m_resume_length=match length: %0
						obj = new Object[] { board.getMatchlength() };
						msg = jibsMessages.convert("m_resume_length", obj);
						showToPlayersWatchers(player, opponent, msg);
						
					}
					// m_resume_points=points for %0: %1
					obj = new Object[] { 
							board.getPlayerXName(), 
							board.getPlayerXScore()
					};
					msg = jibsMessages.convert("m_resume_points", obj);
					showToPlayersWatchers(player, opponent, msg);

					// m_resume_points=points for %0: %1
					obj = new Object[] { 
							board.getPlayerOName(), 
							board.getPlayerOScore()
					};
					msg = jibsMessages.convert("m_resume_points", obj);
					showToPlayersWatchers(player, opponent, msg);
				}

				if (question instanceof InviteQuestion) {
					InviteQuestion inviteQuestion = (InviteQuestion) question;
					matchlength = inviteQuestion.getMatchLength();
					matchVersion = inviteQuestion.getMatchVersion();
					player.stopWatching();
					opponent.stopWatching();
					clipHasChanged(player, opponent);

					game = new JibsGame(
							player,
							opponent,
							matchlength, 0, 0, matchVersion);
					player.setGame(game);
					opponent.setGame(game);
					player.setTogglesForNewGame();
					opponent.setTogglesForNewGame();

					// m_player_joined=** Player %0 has joined you for a %1
					// point match.
					obj = new Object[] { player.getName(), matchlength };
					msg = jibsMessages.convert("m_player_joined", obj);
					opponent.println(msg);

					BackgammonBoard board = game.getBackgammonBoard();
					greet(board);

					JibsNewGameData jibsNewGameData = jibsNewGameData(player,
							opponent);
					// At this point, we have outted "You roll 1, opponent rolled 3"
					// and "opponent makes the first move."
					board.setTurn(jibsNewGameData.getTurn());
					if (jibsNewGameData.getTurn() == BackgammonBoard.turnX) {
						board.setXDie1(jibsNewGameData.getDie1());
						board.setXDie2(jibsNewGameData.getDie2());
					} else {
						board.setODie1(jibsNewGameData.getDie1());
						board.setODie2(jibsNewGameData.getDie2());
					}
					JibsGame.deleteGames(jibsServer,
							jibsNewGameData.getPlayerX(),
							jibsNewGameData.getPlayerO());
					

					// m_start_match=%0 and %1 start a new %2-point match.
					obj = new Object[] {
							jibsNewGameData.getPlayerX().getName(),
							jibsNewGameData.getPlayerO().getName(),
							matchlength
					};
					msg = jibsMessages.convert("m_start_match", obj);
					jibsServer.log(msg);
/*
					int mayDouble = 1;
					if (matchlength == 1)
						mayDouble = 0;
//					jibsNewGameData.getPlayerX().startGame(game, jibsNewGameData.getTurn(),
//							mayDouble, mayDouble);
*/
					game.startGame();
				}
			}
		}
		return true;
	}

	private void showToPlayersWatchers(Player player, Player opponent, String msg) {
		player.println(msg);
		player.show2Watchers(msg);
		opponent.println(msg);
		opponent.show2Watchers(msg);
	}
	
	/** Send the "Starting a new game" message to both players and watchers.
	 * @param player1 One player in the game
	 * @param player2 Another player in the game
	 * @param matchVersion  {@link JibsMatch#nPointMatch} or {@link JibsMatch.unlimitedMatch}
	 * @param matchlength The number of points in the match
	 */
	private void greet(BackgammonBoard board) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		Player playerX = board.getPlayerX();
		Player playerO = board.getPlayerO();
		String playerName = board.getPlayerXName();
		String oppplayerName = board.getPlayerOName();

		// m_start_match_with=Starting a new game with %0.
		Object[] obj = { oppplayerName };
		String msg = jibsMessages.convert("m_start_match_with", obj);
		playerX.println(msg);

		if (board.getMatchVersion().getVersion() == JibsMatch.nPointMatch) {
			// m_start_match_with=Starting a new game with %0.
			obj = new Object[] { playerName };
			msg = jibsMessages.convert("m_start_match_with", obj);
			playerO.println(msg);

			// m_start_match=%0 and %1 start a %2-point match.
			obj = new Object[] {
					playerName, oppplayerName, Integer.valueOf(board.getMatchlength())
			};
			if (board.getPlayerXScore() == 0 && board.getPlayerOScore() == 0) {
				msg = jibsMessages.convert("m_start_match", obj);
				playerX.informPlayers(msg, playerO);
			}
			// message needs to be sent also to both watcher lists
			playerX.informWatchers("m_start_match", obj, true);
			playerO.informWatchers("m_start_match", obj, true);
		} else {
			// m_you_play_unlimited=** You are now playing an unlimited match
			// with %1
			obj = new Object[] { playerName };
			msg = jibsMessages.convert("m_you_play_unlimited", obj);
			playerO.println(msg);
			// m_start_match_unlimited=%0 and %1 start an unlimited match.
			obj = new Object[] { playerName, oppplayerName };
			msg = jibsMessages.convert("m_start_match_unlimited", obj);
			playerX.informPlayers(msg, playerO);
		}
	}

	private JibsNewGameData jibsNewGameData(Player playerX, Player playerO) {
		JibsNewGameData jngd = null;
		Die die1 = new Die();
		Die die2 = new Die();
		int startTurn = determineStartPlayer(playerX, playerO, die1, die2);
		jngd = new JibsNewGameData(playerX, playerO, startTurn,	die1, die2);
		return jngd;
	}

	/** Everybody gets notified of these two player's changed status.
	 * @param player The one player that changed.
	 * @param opponent The other player that changed.
	 */
	private void clipHasChanged(Player player, Player opponent) {
		String ps = player.getWhoinfo();
		String os = opponent.getWhoinfo();
		for (Player p : this.jibsServer.getAllClients().values()) {
			if (p.canCLIP()) {
				p.println(ClipConstants.CLIP_WHO_INFO + " " + ps);
				p.println(ClipConstants.CLIP_WHO_END + " ");
				p.println(ClipConstants.CLIP_WHO_INFO + " " + os);
				p.println(ClipConstants.CLIP_WHO_END + " ");
			}
		}
	}

	private int determineStartPlayer(Player playerX, Player playerO,
			Die rollDice1, Die rollDice2) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();

		String playerName = playerX.getName();
		String oppplayerName = playerO.getName();
		String msg = null;
		Object[] obj = null;

		do {
			rollDice1.roll();
			rollDice2.roll();
			//			rollDice1.setValue(3); // joining Player
			//			rollDice2.setValue(1); // inviting Player

			// m_both_roll=%0 rolled %1, %2 rolled %3
			obj = new Object[] {
					"You", Integer.valueOf(rollDice1.getValue()),
					oppplayerName, Integer.valueOf(rollDice2.getValue())
			};
			msg = jibsMessages.convert("m_both_roll", obj);
			playerX.println(msg);
			// m_both_roll=%0 rolled %1, %2 rolled %3
			obj = new Object[] {
					playerName, Integer.valueOf(rollDice1.getValue()),
					oppplayerName, Integer.valueOf(rollDice2.getValue())
			};
			playerX.informWatchers("m_both_roll", obj, true);
			playerO.informWatchers("m_both_roll", obj, true);
			// m_both_roll=%0 rolled %1, %2 rolled %3
			obj = new Object[] {
					"You", Integer.valueOf(rollDice2.getValue()), playerName,
					Integer.valueOf(rollDice1.getValue())
			};
			msg = jibsMessages.convert("m_both_roll", obj);
			playerO.println(msg);
		} while (rollDice1.getValue() == rollDice2.getValue());

		if (rollDice1.getValue() > rollDice2.getValue()) {
			// m_your_turn=It's your turn to move.
			msg = jibsMessages.convert("m_your_turn");
			playerX.println(msg);
			// m_makes_first_move=%0 makes the first move.
			obj = new Object[] { playerName };
			msg = jibsMessages.convert("m_makes_first_move", obj);
			playerO.println(msg);
			playerX.informWatchers("m_makes_first_move", obj, true);
			playerO.informWatchers("m_makes_first_move", obj, true);

			return BackgammonBoard.turnX;
		} else {
			// m_your_turn=It's your turn to move.
			msg = jibsMessages.convert("m_your_turn");
			playerO.println(msg);
			// m_makes_first_move=%0 makes the first move.
			obj = new Object[] { oppplayerName, };
			msg = jibsMessages.convert("m_makes_first_move", obj);
			playerX.println(msg);
			playerX.informWatchers("m_makes_first_move", obj, true);
			playerO.informWatchers("m_makes_first_move", obj, true);

			return BackgammonBoard.turnO;
		}
	}
}
