/******************************************************************************
 * NewUser_Command.java - Handle registering a new User.
 * $Id: NewUser_Command.java,v 1.7 2011/06/24 01:42:49 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: NewUser_Command.java,v $
 * Revision 1.7  2011/06/24 01:42:49  dick
 * Comment out dead code.
 *
 * Revision 1.6  2011/06/18 06:34:39  dick
 * Initialize to JibsToggle.getDefaultValues() instead of a string of 1's and 0's.
 *
 * Revision 1.5  2011/06/11 19:18:35  dick
 * Stop passing jibsServer all around.  If anybody wants it, they can do JibsServer.getInstance().
 *
 * Revision 1.4  2011/06/06 02:54:21  dick
 * Stop passing the not-so-useful-here Server all around.
 *
 * Revision 1.3  2011/06/05 18:26:33  dick
 * log is a method of JibsServer.  Always write screen logs to the log file.
 * So, no need to pass write-to-logs=true from everywhere.
 *
 * Revision 1.2  2011/05/31 13:02:42  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import java.io.BufferedReader;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.sql.SQLException;
import java.util.Date;
import java.util.StringTokenizer;

import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.Encoder;
import net.sourceforge.jibs.util.JibsToggle;
import net.sourceforge.jibs.util.JibsWriter;

import com.ibatis.sqlmap.client.SqlMapClient;

/** Handle registering a new Player. <br>
 * This is special as it is not derived from JibsCommand
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @since 2004/05/30
 * @version $Revision: 1.7 $ <br> $Date: 2011/06/24 01:42:49 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/NewUser_Command.java">cvs NewUser_Command.java</a>
 */
public class NewUser_Command {
	private BufferedReader in;
	private JibsWriter out;
	private JibsServer jibsServer;

	public NewUser_Command(JibsServer jibsServer, BufferedReader in,
			JibsWriter out) {
		this.in = in;
		this.out = out;
		this.jibsServer = jibsServer;
	}

	public Player createNewUser(JibsServer jibsServer,
			Socket client, String strArgs, String[] args) {
		SqlMapClient sqlMap = jibsServer.getSqlMapInstance();
		Player player = null;

		try {
			out.println("Welcome to JIBS.You just logged in as guest.");
			out.println("Please register before using this server:");

			String strNewUser = chooseName(jibsServer);
			out.println("Your name will be " + strNewUser);

			Password passWord = choosePassword();
			InetAddress iadr = client.getInetAddress();
			player = new Player(null, out, "", -1,
					strNewUser,
					Encoder.encrypt(passWord.getPassWord(), "MD5"),
									1500.0, 0, "", false, new Date(),
									iadr.getHostName());
			// The inital settings
			player.setToggle(JibsToggle.getDefaultValues()); // all boolean toggles
			player.setBoardStyle("2");
			player.setLineLength("0");
			player.setPageLength("0");
			player.setRedoubles("none");
			player.setSortwho("login");
			player.setTimezone("UTC");
			sqlMap.insert("Player.insertPlayer", player);
			player.setPassword(passWord.getPassWord());
			jibsServer.log(
					"User " + strNewUser + "/" +
					passWord.getPassWord() + " registered.");
			out.println("\r\nYou are registered.");
			out.println("Type 'help beginner' to get started.");

			return player;
		} catch (SQLException e1) {
			out.println("\r\n** Your name may only contain letters and the unserscore character _ .");
			jibsServer.logException(e1);
		}
/* Dead code?
		if (player == null) {
			try {
				client.close();
			} catch (IOException e1) {
				jibsServer.logException(e1);
			}
		}
*/
		return player;
	}

	private String chooseName(JibsServer jibsServer) {
		SqlMapClient sqlMap = jibsServer.getSqlMapInstance();
		int count = 0;
		String strNewUser = null;

		boolean bValid = false;
		out.println();
		out.println("Type 'name username' where name is the word 'name' and");
		out.println("username is the login name you want to use.");
		out.println("The username may not contain blanks ' ' or colons ':'.");
		out.println("The system will then ask you for your password twice.");
		out.println("Please make sure that you don't forget your password. All");
		out.println("passwords are encrypted before they are saved. If you forget");
		out.println("your password there is no way to find out what it was.");
		out.println("Please type 'bye' if you don't want to register now.");
		out.println();
		out.println("ONE USERNAME PER PERSON ONLY!!!");

		do {
			String loginCmd = null;

			try {
				loginCmd = in.readLine();
			} catch (IOException e) {
				jibsServer.logException(e);
			}

			StringTokenizer stoken = new StringTokenizer(loginCmd);
			count = 0;

			while (stoken.hasMoreTokens()) {
				String token = stoken.nextToken();

				if ((count == 0) && (token.equalsIgnoreCase("name"))) {
					count++;
				}

				if (count == 1) {
					strNewUser = token;
				}
			}

			// Did the user input name <X> ?
			if (count != 1) {
				out.println();
				out.println("** Your name may only contain letters and the unserscore character _ .");

				continue;
			}

			// check if strNewUser can be chosen
			Integer sqlCount = 0;

			try {
				sqlCount = (Integer) sqlMap.queryForObject("Player.checkPlayer",
						strNewUser);
			} catch (SQLException e) {
				jibsServer.logException(e);
			}

			if (sqlCount > 0) {
				// m_use_another_name=** Please use another name. '%0' is
				// already used by someone else.
				Object[] obj = new Object[] { strNewUser };
				String msg = jibsServer.getJibsMessages()
				.convert("m_use_another_name", obj);
				out.println(msg);
			} else {
				bValid = true;
			}
		} while (!bValid);

		return strNewUser;
	}

	private Password choosePassword() {
		String passWord = "";
		String repeatPassword = "";
		boolean bValid = false;

		do {
			out.println("Type in no password and hit Enter/Return if you want to change it now.");
			out.print("Please give your password: ");
			out.flush();

			try {
				passWord = in.readLine(); // Passwort of <x>
			} catch (IOException e) {
				jibsServer.logException(e);
			}

			if (passWord.length() <= 0) {
				continue;
			}

			out.println("");
			out.print("Please retype your password: ");
			out.flush();

			try {
				repeatPassword = in.readLine(); // Repeated Passwort of <x>
			} catch (IOException e) {
				jibsServer.logException(e);
			}

			if (!passWord.equals(repeatPassword)) {
				out.println("** The two passwords were not identical. Please give them again.");
			} else {
				bValid = true;
			}
		} while (!bValid);

		return (new Password(passWord, repeatPassword));
	}
}


class Password {
	private String passWord;
	private String confirmedPassword;

	public Password(String passWord, String confirmedPassWord) {
		this.passWord = passWord;
		this.confirmedPassword = confirmedPassWord;
	}

	public String getConfirmedPassword() {
		return confirmedPassword;
	}

	public String getPassWord() {
		return passWord;
	}
}
;
